/**
 * @file upnp.js
 * 
 * @brief This file contain implementation of upnp protocol.
 * 
 * @author: mobile2DS
 * 
 * @date: 03.08.2013
 */

// Global constants
// Data type to distinct temp data
var UPNP_MEDIA_INFO = 'media_info';
var UPNP_POS_INFO = 'pos_info';
var UPNP_TRANSPORT_INFO = 'transport_info';
var UPNP_TRANSPORT_SETTINGS_INFO = 'transport_settings_info';
var UPNP_VOLUME_INFO = 'volume_info';

// upnp commands
var UPNP_MUTE = 'set_mute';
var UPNP_PAUSE = 'pause';
var UPNP_UNPAUSE = 'unpause';
var UPNP_RESUME = 'resume';
var UPNP_TOGGLE_PAUSE = 'toggle pause'
var UPNP_GET_MEDIA_URL = 'get_media_url';
var UPNP_GET_MEDIA_URL_POSITION = 'get_media_url_position';
var UPNP_LOAD_CONTENT_AND_PLAY = 'content_and_play';
var UPNP_LOAD_CONTENT_AND_RESUME = 'content_and_play_resume';
var UPNP_PLAY = 'play';
var UPNP_STOP = 'stop';
var UPNP_INC_VOLUME = 'inc_vol';
var UPNP_DEC_VOLUME = 'dec_vol';
var UPNP_VOLUME_STEP = 8;
var UPNP_SET_VOLUME = 'set_volume';
// for testing purposes only
var UPNP_GET_MEDIA_URL_REG_EVENT = 'get_media_reg_event';
var UPNP_STOP_RESUME_PLAY = 'stop_resume_play';
var UPNP_RESUME_ALARM = 'resume_alarm';

var SEEK_ATTEMPTS = 50;

/*
 * ------------------------------------------------------------------------
 * COMMON methods
 */

function sleep(delay) {
	var start = new Date().getTime();
	while (new Date().getTime() < start + delay)
		;
}



function getPlaylistFromText(sString) {
	var oArray=sString.split(';');
	oArray=oArray.splice(0,oArray.length-1);
	lWarn.logln(oArray.join(';'));
	return oArray.join(';')
}

function getVolumeFromText(sString) {
	var oArray=sString.split(';');
	lWarn.logln(oArray[oArray.length-1]);
	return parseInt(oArray[oArray.length-1],10);
}


/**
 * @brief Save temp data
 * 
 * @param oTempData -
 *            holds temp data for saving in format: 1.) player id 2.) action id
 *            3.) action type: present == 1, or activity == 0 4.) data type:
 *            custom defined 5.) array holding values
 * 
 * @return Returns true if new mode for room is assigned, else false
 */
function saveTempData(nPlayerID, nActionID, nActionType, szDataType, oTempData) {
	try {
		var tmp_data_root = 'upnp/tempData';
		// Check if same temp data already exists

		// Write new temp data
		var path_prefix = nPlayerID + '/' + szDataType;

		// if (!bTempDataExists)
		{
			// Store player ID
			storeSubParameter(tmp_data_root, path_prefix + '/player_id', nPlayerID);
			// Store action ID
			storeSubParameter(tmp_data_root, path_prefix + '/action_id', nActionID);
			// Store action type (present == 1, activity == 0)
			storeSubParameter(tmp_data_root, path_prefix + '/action_type', nActionType);

			for (i = 0; i < oTempData.length; i++) {
				lInfo.logln(i + ': ' + oTempData[i]);
				storeSubParameter(tmp_data_root, path_prefix + '/temp' + i, oTempData[i]);
			}

			Property.store();

			lInfo.logln("Temp data stored");
		}
	} catch (e) {
		lWarn.logln("Error by saving temp data: " + e);
		for ( var i in e) {
			lWarn.logln(i + ': ' + e[i]);
		}
	}
}

function loadTempData(nPlayerID, nActionID, nActionType, szDataType, oTempData) {
	var idValue = -1;
	var tmp_data_root = 'upnp/tempData';
	var path_appendix = nPlayerID + '/' + szDataType;
	var full_path = tmp_data_root + '/' + path_appendix;

	lInfo.logln('Search for temp data in: ' + full_path);

	var baseNode = Property.getNode(full_path);
	if (baseNode != null) {
		var oArrayNode = baseNode.getChildren();
		if ((oArrayNode != null) && (oArrayNode.length > 3)) {
			lInfo.logln(oArrayNode[1].getValue());

			if ((oArrayNode[0].getValue() == nPlayerID) && (oArrayNode[1].getValue() == nActionID) && (oArrayNode[2].getValue() == nActionType)) {
				lInfo.logln('Fill output array with temp values');
				// fill the array
				for ( var i = 0; i < oArrayNode.length - 3; i++) {
					oTempData.push(oArrayNode[i + 3].getValue());
				}
			}
		} else {
			lWarn.logln('Read array is not valid');
		}
	}

	return idValue;
}

/**
 * @brief Parse UPNP XML response and return value for specific field
 * 
 * @param szXMLRespond -
 *            respond from upnp device in XML syntax
 * @param szXMLFieldName -
 *            searched field name enclosing the value
 * @param szContainsText -
 *            filter that found value must contain this text. if '' no filter
 *            applying
 * 
 * @return
 */
function upnpParseXMLRespond(szXMLRespond, szXMLFieldName, szContainsText) {
	// lInfo.logln(szXMLRespond);
	var searched_value = '';
	var found = 0;
	var xml_text = '';
	var start_pos = 0;

	xml_text = szXMLRespond;

	do {
		start_pos = xml_text.indexOf('<' + szXMLFieldName);

		if (start_pos == -1) {
			break;
		}

		start_pos += szXMLFieldName.length + 1;
		start_pos += 1 /* skip '>' */;

		var end_pos = xml_text.indexOf('</' + szXMLFieldName);

		if (end_pos == -1) {
			break;
		}
		;

		searched_value = xml_text.substring(start_pos, end_pos);

		if (szContainsText != "") {
			if (searched_value.indexOf(szContainsText) == -1) {
				start_pos = end_pos + szXMLFieldName.length + 1;
				xml_text = xml_text.substring(start_pos, xml_text.length);

				lInfo.logln("Filter not match, continue search: : " + xml_text);
			} else {
				found = 1;
			}
		} else {
			found = 1;
		}
	} while ((szContainsText != "") && (found == 0) && (xml_text != ""))

	lInfo.logln("XML parser found: " + searched_value);

	return searched_value;
}

function upnpResumeMusic(szIp, nPort, nValue, nPlayerId, szAVTransportService, oCurlOptions, sResumeFile, sSeek) {
	var result = new Object({
		'status' : '',
		'data' : ''
	});

	// resume music from playlist
	lInfo.logln("upnp-controller after timeout:" + sResumeFile);
	if (sResumeFile != "") {
		upnpSetContentSrc(nPlayerId, szIp, nPort, szAVTransportService, sResumeFile, result, oCurlOptions);
		upnpPlay(nPlayerId, szIp, nPort, szAVTransportService, result, oCurlOptions);
		lInfo.logln("upnp-controller seek:" + sSeek);

		if (sSeek != "") {
			lInfo.logln("upnp-controller inside seek:" + sSeek);
			var bSeek = false;
			var nAttemps = SEEK_ATTEMPTS;
			while (!bSeek) {
				sInfo = upnpGetTransportInfo(nPlayerId, szIp, nPort, szAVTransportService, nValue, result, oCurlOptions);
				lInfo.logln("upnp-controller attempt:" + nAttemps + " / " + sInfo);
				if (sInfo == 'PLAYING') {
					upnpSeek(nPlayerId, szIp, nPort, sSeek, szAVTransportService, result, oCurlOptions);
					bSeek = true;
				}

				nAttemps = nAttemps - 1;
				if (!bSeek && nAttemps < 1)
					bSeek = true;
				sleep(200);
			}
		}
	} else { // stop
		upnpStop(nPlayerId, szIp, nPort, szAVTransportService, result, oCurlOptions);
	}

	// reseting scene priority
	lInfo.logln("reseting scene priority on resume play!");
	storeSubParameter('upnp', 'activeScenePriority', 0);
	Property.store();
}

/*
 * ---------------------------------------------------------------------------
 * UPNP related METHODS
 */

/**
 * @brief Execute UPNP command
 * 
 * @param
 * 
 * @return
 */

function executeUpnpCmd(szCmd, szDeviceType, szIp, nPort, nValue, szMediaContentURL, nPlayerId) {
	var result = new Object({
		'status' : '',
		'data' : ''
	});

	var curl_options = new Object({
		'debug' : 1,
		'connectTimeout' : 10
	/* in seconds */
	});

	lInfo.logln("executeUpnpCmd: " + szCmd + " player id: " + nPlayerId + " ip addr: " + szIp + " port: " + nPort + " device type: " + szDeviceType);
	var avtransport_service_type = '/AVTransport/ctrl';
	if (szDeviceType == 'SONOS') {
		avtransport_service_type = '/MediaRenderer/AVTransport/Control';
	} else if (szDeviceType == 'SAMSUNGTV') {
		avtransport_service_type = '/AVTransport';
	} else if (szDeviceType == 'BUBBLE') {
		avtransport_service_type = '/AVTransport/action';
	}

	var render_service_type = '/RenderingControl/ctrl';
	if (szDeviceType == 'SONOS') {
		render_service_type = '/MediaRenderer/RenderingControl/Control';
	} else if (szDeviceType == 'SAMSUNGTV') {
		render_service_type = '/RenderingControl';
	} else if (szDeviceType == 'BUBBLE') {
		render_service_type = '/RenderingControl/action';
	}

	if (szCmd == UPNP_MUTE) {
		lInfo.logln("upnp-controller exec setMute");
		upnpSetMute(nPlayerId, szIp, nPort, nValue, render_service_type, result, curl_options);
		// upnpSetMute1(ip, port, value);
	} else if (szCmd == UPNP_TOGGLE_PAUSE) {	
		sInfo = upnpGetTransportInfo(nPlayerId, szIp, nPort, avtransport_service_type, nValue, result, curl_options);
		if (sInfo == "PLAYING") {
			oCurrent = upnpGetPositionInfo(nPlayerId, szIp, nPort, avtransport_service_type, nValue, result);
			storeSubParameter('activity', nPlayerId + '/pauseTrack', oCurrent.track);
			storeSubParameter('activity', nPlayerId + '/pauseTrackPos', oCurrent.position);
			Property.store();
			upnpSetPause(nPlayerId, szIp, nPort, nValue, avtransport_service_type, result, curl_options);
		} else {
			lInfo.logln("upnp-controller exec unpause");
			var sInfo = upnpGetTransportInfo(nPlayerId, szIp, nPort, avtransport_service_type, nValue, result, curl_options);
			upnpStop(nPlayerId, szIp, nPort, avtransport_service_type, result, curl_options);

			var sResumeFrom = loadSubParameter('activity', nPlayerId + '/pauseTrack');
			var sResumeSeek = loadSubParameter('activity', nPlayerId + '/pauseTrackPos');
			if (sResumeFrom != '') {
				var timeoutHandle;
				timeoutHandle = loadSubParameter('upnp', 'timer');
				lInfo.logln("UnpauseFrom:" + sResumeFrom);
				timeoutHandle = setTimeout(function() {
					upnpResumeMusic(szIp, nPort, nValue, nPlayerId, avtransport_service_type, curl_options, sResumeFrom, sResumeSeek)
				}, nValue);

				// Store timeoutHandle to the property
				storeSubParameter('upnp', 'timer', timeoutHandle);
				Property.store();
			} else {
				// only load previous music
				if (szMediaContentURL.split(';').length>1)
				{
					upnpSetVolume(nPlayerId, szIp, nPort, render_service_type, getVolumeFromText(szMediaContentURL), result, curl_options);
					upnpSetContentSrc(nPlayerId, szIp, nPort, avtransport_service_type, getPlaylistFromText(szMediaContentURL), result, curl_options);
				} else
					upnpSetContentSrc(nPlayerId, szIp, nPort, avtransport_service_type, szMediaContentURL, result, curl_options);
			}
			// clear remembered pause panic event state
			storeSubParameter('activity', nPlayerId + '/pauseTrack', '');
			storeSubParameter('activity', nPlayerId + '/pauseTrackPos', '');
			Property.store();			
		}		
	} else if (szCmd == UPNP_PAUSE) {
		lInfo.logln("upnp-controller exec Pause");
		if (nValue == 1) { // alarms - remember current playsong
			sInfo = upnpGetTransportInfo(nPlayerId, szIp, nPort, avtransport_service_type, nValue, result, curl_options);
			if (sInfo == "PLAYING") {
				oCurrent = upnpGetPositionInfo(nPlayerId, szIp, nPort, avtransport_service_type, nValue, result);
				storeSubParameter('activity', nPlayerId + '/pauseTrack', oCurrent.track);
				storeSubParameter('activity', nPlayerId + '/pauseTrackPos', oCurrent.position);
				Property.store();
				lInfo.logln("Pause - remembering current track info: " + oCurrent.track);
			}
		}
		upnpSetPause(nPlayerId, szIp, nPort, nValue, avtransport_service_type, result, curl_options);
	} else if (szCmd == UPNP_RESUME) {
		lInfo.logln("upnp-controller exec Resume");
		upnpSetResume(nPlayerId, szIp, nPort, nValue, avtransport_service_type, result, curl_options);
	} else if (szCmd == UPNP_STOP) {
		lInfo.logln("upnp-controller exec Stop - defVolume" + nValue);
		upnpStop(nPlayerId, szIp, nPort, avtransport_service_type, result, curl_options);
		if (nValue > -1)
			upnpSetVolume(nPlayerId, szIp, nPort, render_service_type, nValue, result, curl_options);
	} else if (szCmd == UPNP_LOAD_CONTENT_AND_PLAY) {
		lInfo.logln("upnp-controller exec LoadContentAndPlay");
		if (nValue == 1) { // alarms - remember current playsong
			sPlay = Property.getProperty("/scripts/upnp-controller/activity/" + nPlayerId + "/lastAlarmTrack");
			if (sPlay == '') { // store only if no other event has stored
				// before this
				sInfo = upnpGetTransportInfo(nPlayerId, szIp, nPort, avtransport_service_type, nValue, result, curl_options);
				oCurrent = upnpGetPositionInfo(nPlayerId, szIp, nPort, avtransport_service_type, nValue, result);
				storeSubParameter('activity', nPlayerId + '/lastAlarmTrack', oCurrent.track);
				storeSubParameter('activity', nPlayerId + '/lastAlarmTrackPos', oCurrent.position);
				storeSubParameter('activity', nPlayerId + '/lastAlarmTrackState', sInfo); // PLAYING,
				// STOPPED...
				Property.store();
				lInfo.logln("Store LastAlarm " + oCurrent.track);
			}
		}
		upnpStop(nPlayerId, szIp, nPort, avtransport_service_type, result, curl_options);

		if (szMediaContentURL.split(';').length>1)
		{
			upnpSetVolume(nPlayerId, szIp, nPort, render_service_type, getVolumeFromText(szMediaContentURL), result, curl_options);
			upnpSetContentSrc(nPlayerId, szIp, nPort, avtransport_service_type, getPlaylistFromText(szMediaContentURL), result, curl_options);
		} else
			upnpSetContentSrc(nPlayerId, szIp, nPort, avtransport_service_type, szMediaContentURL, result, curl_options);

		
		upnpPlay(nPlayerId, szIp, nPort, avtransport_service_type, result, curl_options);
	} else if (szCmd == UPNP_UNPAUSE) {
		lInfo.logln("upnp-controller exec unpause");
		var sInfo = upnpGetTransportInfo(nPlayerId, szIp, nPort, avtransport_service_type, nValue, result, curl_options);
		upnpStop(nPlayerId, szIp, nPort, avtransport_service_type, result, curl_options);

		var sResumeFrom = loadSubParameter('activity', nPlayerId + '/pauseTrack');
		var sResumeSeek = loadSubParameter('activity', nPlayerId + '/pauseTrackPos');
		if (sResumeFrom != '') {
			var timeoutHandle;
			timeoutHandle = loadSubParameter('upnp', 'timer');
			lInfo.logln("UnpauseFrom:" + sResumeFrom);
			timeoutHandle = setTimeout(function() {
				upnpResumeMusic(szIp, nPort, nValue, nPlayerId, avtransport_service_type, curl_options, sResumeFrom, sResumeSeek)
			}, nValue);

			// Store timeoutHandle to the property
			storeSubParameter('upnp', 'timer', timeoutHandle);
			Property.store();
		} else {
			// only load previous music
			var szMediaURL=szMediaContentURL;
			if (szMediaContentURL.split(';').length>1)
			{
				upnpSetVolume(nPlayerId, szIp, nPort, render_service_type, getVolumeFromText(szMediaContentURL), result, curl_options);
				upnpSetContentSrc(nPlayerId, szIp, nPort, avtransport_service_type, getPlaylistFromText(szMediaContentURL), result, curl_options);
			} else
				upnpSetContentSrc(nPlayerId, szIp, nPort, avtransport_service_type, szMediaContentURL, result, curl_options);
		}
		// clear remembered pause panic event state
		storeSubParameter('activity', nPlayerId + '/pauseTrack', '');
		storeSubParameter('activity', nPlayerId + '/pauseTrackPos', '');
		Property.store();
	} else if (szCmd == UPNP_RESUME_ALARM) {
		lInfo.logln("upnp-controller exec Resume from undo alarm");
		var sInfo = upnpGetTransportInfo(nPlayerId, szIp, nPort, avtransport_service_type, nValue, result, curl_options);
		// if (sInfo == 'PLAYING')
		upnpStop(nPlayerId, szIp, nPort, avtransport_service_type, result, curl_options);

		var sResumeFrom = loadSubParameter('activity', nPlayerId + '/lastAlarmTrackPos');
		var sResumeState = loadSubParameter('activity', nPlayerId + '/lastAlarmTrackState');
		if (sResumeState == 'PLAYING') {
			lInfo.logln("Set the timeout interval when playlist must be resumed");
			var timeoutHandle;
			// Fetch timeoutHandle from the property
			timeoutHandle = loadSubParameter('upnp', 'timer');

			lInfo.logln("ResumeFrom:" + sResumeFrom);
			timeoutHandle = setTimeout(function() {
				if (szMediaContentURL.split(';').length>1)
				{
					upnpSetVolume(nPlayerId, szIp, nPort, render_service_type, getVolumeFromText(szMediaContentURL), result, curl_options);
					upnpSetContentSrc(nPlayerId, szIp, nPort, avtransport_service_type, getPlaylistFromText(szMediaContentURL), result, curl_options);
				} else
					upnpSetContentSrc(nPlayerId, szIp, nPort, avtransport_service_type, szMediaContentURL, result, curl_options);
			}, nValue);

			// Store timeoutHandle to the property
			storeSubParameter('upnp', 'timer', timeoutHandle);
			Property.store();
		} else {
			// only load previous music
			if (szMediaContentURL.split(';').length>1)
			{
				upnpSetVolume(nPlayerId, szIp, nPort, render_service_type, getVolumeFromText(szMediaContentURL), result, curl_options);
				upnpSetContentSrc(nPlayerId, szIp, nPort, avtransport_service_type, getPlaylistFromText(szMediaContentURL), result, curl_options);
			} else
				upnpSetContentSrc(nPlayerId, szIp, nPort, avtransport_service_type, szMediaContentURL, result, curl_options);
		}
		// clear remembered resum panic event state
		storeSubParameter('activity', nPlayerId + '/lastAlarmTrack', '');
		storeSubParameter('activity', nPlayerId + '/lastAlarmTrackPos', '');
		storeSubParameter('activity', nPlayerId + '/lastAlarmTrackState', '');
		Property.store();
	} else if (szCmd == UPNP_LOAD_CONTENT_AND_RESUME) {
		lInfo.logln("upnp-controller exec " + UPNP_LOAD_CONTENT_AND_RESUME);
		var sInfo = upnpGetTransportInfo(nPlayerId, szIp, nPort, avtransport_service_type, nValue, result, curl_options);
		var sResumeFile = "";
		var sSeek = "";
		if (sInfo == 'PLAYING') {
			// /upnpSeek(nPlayerId, szIp, nPort, 888, avtransport_service_type,
			// 'TRACK_NR');
			var oCurrent = upnpGetPositionInfo(nPlayerId, szIp, nPort, avtransport_service_type, nValue, result, curl_options);
			var volume = upnpGetVolume(nPlayerId, szIp, nPort, render_service_type, result, curl_options);
						
			lInfo.logln("upnp-controller exec track:" + oCurrent.track + " / position:" + oCurrent.position +" / Volume: " + volume);
			if (oCurrent.track != "") {
				sResumeFile = oCurrent.track;
			}
			if (oCurrent.position != "") {
				sSeek = oCurrent.position;
			}
		} // if (sInfo == 'PLAYING')

		// playing event music
		lInfo.logln("upnp-controller scene play:" + szMediaContentURL);
		upnpStop(nPlayerId, szIp, nPort, avtransport_service_type, result, curl_options);
		if (szMediaContentURL.split(';').length>1)
		{
			upnpSetVolume(nPlayerId, szIp, nPort, render_service_type, getVolumeFromText(szMediaContentURL), result, curl_options);
			upnpSetContentSrc(nPlayerId, szIp, nPort, avtransport_service_type, getPlaylistFromText(szMediaContentURL), result, curl_options);
		} else
			upnpSetContentSrc(nPlayerId, szIp, nPort, avtransport_service_type, szMediaContentURL, result, curl_options);

		upnpPlay(nPlayerId, szIp, nPort, avtransport_service_type, result, curl_options);

		if (nValue < 1)
			nValue = 1000;
		else
			nValue = nValue * 1000;

		var timeoutHandle;
		// Fetch timeoutHandle from the property
		timeoutHandle = loadSubParameter('upnp', 'timer');

		timeoutHandle = setTimeout(function() {
			upnpSetVolume(nPlayerId, szIp, nPort, render_service_type, parseInt(""+volume,10), result, curl_options);
			upnpResumeMusic(szIp, nPort, nValue, nPlayerId, avtransport_service_type, curl_options, sResumeFile, sSeek);
		}, nValue);

		// Store timeoutHandle to the property
		storeSubParameter('upnp', 'timer', timeoutHandle);
		Property.store();
	} else if (szCmd == UPNP_SET_VOLUME) {
		lInfo.logln("upnp-controller exec SetVolume");
		upnpSetVolume(nPlayerId, szIp, nPort, render_service_type, nValue, result, curl_options);
	} else if (szCmd == UPNP_GET_MEDIA_URL_REG_EVENT) {
		lInfo.logln("upnp-controller exec GetMediaRegEvent");
		result.data = upnpGetMediaContent(nPlayerId, szIp, nPort, avtransport_service_type, result, curl_options);
	} else if (szCmd === UPNP_GET_MEDIA_URL) {
		lInfo.logln("upnp-controller exec GetMedia");
		upnpGetMediaContent(nPlayerId, szIp, nPort, avtransport_service_type, result, curl_options);
	} else if (szCmd == UPNP_STOP_RESUME_PLAY) {
		lInfo.logln("upnp-controller exec GetMediaRegEvent");
		upnpGetMediaInfo(nPlayerId, szIp, nPort, avtransport_service_type, result, curl_options);
		upnpGetPositionInfo(nPlayerId, szIp, nPort, avtransport_service_type, nValue, result, curl_options);
		upnpGetTransportInfo(nPlayerId, szIp, nPort, avtransport_service_type, nValue, result, curl_options);
		upnpGetTransportSettings(nPlayerId, szIp, nPort, avtransport_service_type, nValue, result, curl_options);
		/*
		 * upnpStop(nPlayerId, szIp, nPort, avtransport_service_type, result ,
		 * curl_options); upnpPlayStoredPlaylist(nPlayerId, szIp, nPort,
		 * avtransport_service_type , curl_options); upnpSetTimeout(nPlayerId,
		 * szIp, nPort, avtransport_service_type , curl_options);
		 */
	} else if (szCmd == UPNP_GET_MEDIA_URL_POSITION) {
		upnpGetPositionTrackUrl(nPlayerId, szIp, nPort, avtransport_service_type, result, curl_options);
	} else if (szCmd == UPNP_INC_VOLUME) {
		lInfo.logln(nPlayerId + "/" + szIp + "/" + nPort + "/" + avtransport_service_type + "/" + result);
		upnpSetPlayerVolume(1, nPlayerId, szIp, nPort, render_service_type, result, curl_options);
	} else if (szCmd == UPNP_DEC_VOLUME) {
		upnpSetPlayerVolume(0, nPlayerId, szIp, nPort, render_service_type, result, curl_options);
	}

	return result;
}

function upnpSetPlayerVolume(nVolumeType, nPlayerId, szIpAddressRenderer, nPortRenderer, szServiceType, oResult, oCurlOptions) {

	try {
		var http_headers = [];
		http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
		http_headers['extra_hdr2'] = 'SOAPAction: ' + '"urn:schemas-upnp-org:service:' + 'RenderingControl' + ':1'
		// + szServiceType + ':1'
		+ '#' + 'GetVolume"'
		var h = new HTTP(oCurlOptions, http_headers);

		h.reset();

		var url = 'http://' + szIpAddressRenderer + ':' + nPortRenderer + szServiceType;

		var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
				+ '<s:Body>' + '<u:GetVolume xmlns:u="urn:schemas-upnp-org:service:' + 'RenderingControl' + ':1' + '">' + '<InstanceID>0</InstanceID>' + '<Channel>Master</Channel>' + '</u:GetVolume>'
				+ '</s:Body>' + '</s:Envelope>';

		var result = h.post(url, '', action);
		var nVolume = parseInt(upnpParseXMLRespond(result.body, 'CurrentVolume', ''));
		lInfo.logln("StartVolume:" + nVolume);

		var bChangeVolume = true;
		if (nVolumeType == 1) { // increase
			if (nVolume > 99)
				bChangeVolume = false;
			nVolume = nVolume + UPNP_VOLUME_STEP;
			if (nVolume > 100)
				nVolume = 100;
		} else { // decrease
			if (nVolume < 1)
				bChangeVolume = false;
			nVolume = nVolume - UPNP_VOLUME_STEP;
			if (nVolume < 0)
				nVolume = 0;
		}

		if (bChangeVolume) {
			upnpSetVolume(nPlayerId, szIpAddressRenderer, nPortRenderer, szServiceType, nVolume, result);
		}

		lInfo.logln(url);
		lInfo.logln(action);
	} catch (e) {
		lWarn.logln("Error executing upnpSetVolume command " + e);
	}
}

function upnpGetProtocolInfo(nPlayerId, szIpAddressRenderer, nPortRenderer, nValue, szServiceType, oResult, oCurlOptions) {

	try {
		var http_headers = [];
		http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
		http_headers['extra_hdr2'] = 'SOAPAction: ' + '"urn:schemas-upnp-org:service:' + 'ConnectionManager' + ':1' + '#' + 'GetProtocolInfo"'
		var h = new HTTP(oCurlOptions, http_headers);

		h.reset();

		var url = 'http://' + szIpAddressRenderer + ':' + nPortRenderer + '/ConnectionManager/ctrl';

		var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
				+ '<s:Body>' + '<u:GetProtocolInfo xmlns:u="urn:schemas-upnp-org:service:' + 'ConnectionManager' + ':1' + '">' + '</s:Body>' + '</s:Envelope>';

		var result = h.post(url, '', action);
	} catch (e) {
		lWarn.logln("Error executing upnpGetProtocolInfo command " + e);
	}
}

/**
 * @brief Set mute
 * 
 * @param nPlayerId -
 *            player ID
 * @param szIpAddressRenderer -
 *            IP address of the UPNP renderer
 * @param nPortRenderer -
 *            Port number of the UPNP renderer
 * @param nValue -
 *            0 -> perform unmute, 1 -> perform mute
 * @param szServiceType -
 *            Service type
 * @param oResult -
 *            holds result and returned data
 * @param oCurlOptions -
 *            additional options for curl used by HTTP
 * 
 * @return
 */
function upnpSetMute(nPlayerId, szIpAddressRenderer, nPortRenderer, nValue, szServiceType, oResult, oCurlOptions) {

	try {
		var http_headers = [];
		http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
		http_headers['extra_hdr2'] = 'SOAPAction: ' + '"urn:schemas-upnp-org:service:' + 'RenderingControl' + ':1'
		// + szServiceType + ':1'
		+ '#' + 'SetMute"'
		var h = new HTTP(oCurlOptions, http_headers);

		h.reset();

		var url = 'http://' + szIpAddressRenderer + ':' + nPortRenderer + szServiceType;

		var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
				+ '<s:Body>' + '<u:SetMute xmlns:u="urn:schemas-upnp-org:service:' + 'RenderingControl' + ':1' + '">' + '<InstanceID>0</InstanceID>' + '<Channel>Master</Channel>' + '<DesiredMute>'
				+ nValue + '</DesiredMute>' + '</u:SetMute>' + '</s:Body>' + '</s:Envelope>';

		var result = h.post(url, '', action);
		lInfo.logln(url);
		lInfo.logln(action);
	} catch (e) {
		lWarn.logln("Error executing upnpSetMute command " + e);
	}
}

/**
 * @brief Set pause
 * 
 * @param nPlayerId -
 *            player ID
 * @param szIpAddressRenderer -
 *            IP address of the UPNP renderer
 * @param nPortRenderer -
 *            Port number of the UPNP renderer
 * @param nValue -
 *            not used
 * @param szServiceType -
 *            Service type
 * @param oResult -
 *            holds result and returned data
 * @param oCurlOptions -
 *            additional options for curl used by HTTP
 * 
 * @return
 */
function upnpSetPause(nPlayerId, szIpAddressRenderer, nPortRenderer, nValue, szServiceType, oResult, oCurlOptions) {

	try {
		lInfo.logln("PAUSE STARTING");

		var http_headers = [];
		http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
		http_headers['extra_hdr2'] = 'SOAPAction: ' + '"urn:schemas-upnp-org:service:' + 'AVTransport' + ':1' + '#' + 'Pause"'
		var h = new HTTP(oCurlOptions, http_headers);

		h.reset();

		var url = 'http://' + szIpAddressRenderer + ':' + nPortRenderer + szServiceType;

		var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
				+ '<s:Body>' + '<u:Pause xmlns:u="urn:schemas-upnp-org:service:' + 'AVTransport' + ':1' + '">' + '<InstanceID>0</InstanceID>' + '</u:Pause>' + '</s:Body>' + '</s:Envelope>';

		var result = h.post(url, '', action);
	} catch (e) {
		lWarn.logln("Error executing upnpSetPause command " + e);
	}
}

function upnpSeek(nPlayerId, szIpAddressRenderer, nPortRenderer, nValue, szServiceType, oResult, oCurlOptions) {

	try {
		var http_headers = [];
		http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
		http_headers['extra_hdr2'] = 'SOAPAction: ' + '"urn:schemas-upnp-org:service:' + 'AVTransport' + ':1' + '#' + 'Seek"'
		var h = new HTTP(oCurlOptions, http_headers);

		h.reset();

		var url = 'http://' + szIpAddressRenderer + ':' + nPortRenderer + szServiceType;

		var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
				+ '<s:Body>' + '<u:Seek xmlns:u="urn:schemas-upnp-org:service:' + 'AVTransport' + ':1' + '">' + '<InstanceID>0</InstanceID>' + '<Unit>REL_TIME</Unit>' + '<Target>' + nValue
				+ '</Target>' + '</u:Seek>' + '</s:Body>' + '</s:Envelope>';

		var result = h.post(url, '', action);
	} catch (e) {
		lWarn.logln("Error executing upnpSetPause command " + e);
	}
}

/**
 * @brief Set resume
 * 
 * @param nPlayerId -
 *            player ID
 * @param szIpAddressRenderer -
 *            IP address of the UPNP renderer
 * @param nPortRenderer -
 *            Port number of the UPNP renderer
 * @param nValue -
 *            not used
 * @param szServiceType -
 *            Service type
 * @param oResult -
 *            holds result and returned data
 * @param oCurlOptions -
 *            additional options for curl used by HTTP
 * 
 * @return
 */
function upnpSetResume(nPlayerId, szIpAddressRenderer, nPortRenderer, nValue, szServiceType, oResult, oCurlOptions) {

	try {
		var result = upnpPlay(szIpAddressRenderer, nPortRenderer, szServiceType, oCurlOptions);
	} catch (e) {
		lWarn.logln("Error executing upnpSetResume command " + e);
	}
}

/**
 * @brief Perform stop
 * 
 * @param nPlayerId -
 *            player ID
 * @param szIpAddressRenderer -
 *            IP address of the UPNP renderer
 * @param nPortRenderer -
 *            Port number of the UPNP renderer
 * @param szServiceType -
 *            Service type
 * @param oResult -
 *            holds result and returned data
 * @param oCurlOptions -
 *            additional options for curl used by HTTP
 * 
 * @return
 */
function upnpStop(nPlayerId, szIpAddressRenderer, nPortRenderer, szServiceType, oResult, oCurlOptions) {

	try {
		var http_headers = [];
		http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
		http_headers['extra_hdr2'] = 'SOAPAction:' + '"urn:schemas-upnp-org:service:' + 'AVTransport' + ':1#' + 'Stop"';

		var h = new HTTP(oCurlOptions, http_headers);

		h.reset();

		var url = 'http://' + szIpAddressRenderer + ':' + nPortRenderer + szServiceType;

		var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
				+ '<s:Body>' + '<u:Stop xmlns:u="urn:schemas-upnp-org:service:AVTransport:1">' + '<InstanceID>0</InstanceID>' + '</u:Stop>' + '</s:Body>' + '</s:Envelope>';

		var result = h.post(url, '', action);
	} catch (e) {
		lWarn.logln("Error executing upnpStop command " + e + " [url=" + url + "]");
	}
}

/**
 * @brief Perform upnp server source selection
 * 
 * @param nPlayerId -
 *            player ID
 * @param szIpAddressRenderer -
 *            IP address of the UPNP renderer
 * @param nPortRenderer -
 *            Port number of the UPNP renderer
 * @param szServiceType -
 *            Service type
 * @param szUpnpServerContent -
 *            full path to the upnp server content
 * @param oResult -
 *            holds result and returned data
 * @param oCurlOptions -
 *            additional options for curl used by HTTP
 * 
 * @return
 */
function upnpSetContentSrc(nPlayerId, szIpAddressRenderer, nPortRenderer, szServiceType, szUpnpServerContent, oResult, oCurlOptions) {

	try {
		var http_headers = [];
		http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
		http_headers['extra_hdr2'] = 'SOAPAction:' + '"urn:schemas-upnp-org:service:' + 'AVTransport' + ':1#' + 'SetAVTransportURI"';

		var h = new HTTP(oCurlOptions, http_headers);

		h.reset();

		var url = 'http://' + szIpAddressRenderer + ':' + nPortRenderer + szServiceType;

		var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
				+ '<s:Body>' + '<u:SetAVTransportURI xmlns:u="urn:schemas-upnp-org:service:AVTransport:1">' + '<InstanceID>0</InstanceID>' + '<CurrentURI>' + szUpnpServerContent + '</CurrentURI>'
				+ '<CurrentURIMetaData />' + '</u:SetAVTransportURI>' + '</s:Body>' + '</s:Envelope>';

		var result = h.post(url, '', action);

		lInfo.logln(url);
		lInfo.logln(action);
	} catch (e) {
		lWarn.logln("Error executing upnpSetContentSrc command " + e);
	}
}

/**
 * @brief Perform play
 * 
 * @param nPlayerId -
 *            player ID
 * @param szIpAddressRenderer -
 *            IP address of the UPNP renderer
 * @param nPortRenderer -
 *            Port number of the UPNP renderer
 * @param szServiceType -
 *            Service type
 * @param oResult -
 *            holds result and returned data
 * @param oCurlOptions -
 *            additional options for curl used by HTTP
 * 
 * @return
 */
function upnpPlay(nPlayerId, szIpAddressRenderer, nPortRenderer, szServiceType, oResult, oCurlOptions) {

	try {
		var http_headers = [];
		http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
		http_headers['extra_hdr2'] = 'SOAPAction:' + '"urn:schemas-upnp-org:service:' + 'AVTransport' + ':1#' + 'Play"';

		var h = new HTTP(oCurlOptions, http_headers);

		h.reset();

		var url = 'http://' + szIpAddressRenderer + ':' + nPortRenderer + szServiceType;

		var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
				+ '<s:Body>' + '<u:Play xmlns:u="urn:schemas-upnp-org:service:AVTransport:1">' + '<InstanceID>0</InstanceID>' + '<Speed>1</Speed>' + '</u:Play>' + '</s:Body>' + '</s:Envelope>';

		var result = h.post(url, '', action);
	} catch (e) {
		lWarn.logln("Error executing upnpPlay command " + e);
	}
}

/**
 * @brief Perform upnp set volume
 * 
 * @param nPlayerId -
 *            player ID
 * @param szIpAddressRenderer -
 *            IP address of the UPNP renderer
 * @param nPortRenderer -
 *            Port number of the UPNP renderer
 * @param szServiceType -
 *            Service type
 * @param nVolumeLevel -
 *            volume level
 * @param oResult -
 *            holds result and returned data
 * @param oCurlOptions -
 *            additional options for curl used by HTTP
 * 
 * @return
 */
function upnpSetVolume(nPlayerId, szIpAddressRenderer, nPortRenderer, szServiceType, nVolumeLevel, oResult, oCurlOptions) {

	try {
		var http_headers = [];
		http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
		http_headers['extra_hdr2'] = 'SOAPAction:' + '"urn:schemas-upnp-org:service:' + 'RenderingControl' + ':1#' + 'SetVolume"';

		var h = new HTTP(oCurlOptions, http_headers);

		h.reset();

		var url = 'http://' + szIpAddressRenderer + ':' + nPortRenderer + szServiceType;

		var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
				+ '<s:Body>' + '<u:SetVolume xmlns:u="urn:schemas-upnp-org:service:RenderingControl:1">' + '<InstanceID>0</InstanceID>' + '<Channel>Master</Channel>' + '<DesiredVolume>'
				+ nVolumeLevel + '</DesiredVolume>' + '</u:SetVolume>' + '</s:Body>' + '</s:Envelope>';

		var result = h.post(url, '', action);

		lInfo.logln(url);
		lInfo.logln(action);
	} catch (e) {
		lWarn.logln("Error executing upnpSetContentSrc command " + e);
	}
}

/**
 * @brief Perform get currently played media content src from upnp renderer
 * 
 * @param nPlayerId -
 *            player ID
 * @param szIpAddressRenderer -
 *            IP address of the UPNP renderer
 * @param nPortRenderer -
 *            Port number of the UPNP renderer
 * @param szServiceType -
 *            Service type
 * @param oResult -
 *            holds result and returned data
 * @param oCurlOptions -
 *            additional options for curl used by HTTP
 * 
 * @return
 */
function upnpGetMediaContent(nPlayerId, szIpAddressRenderer, nPortRenderer, szServiceType, oResult, oCurlOptions) {
	try {
		var http_headers = [];
		http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
		http_headers['extra_hdr2'] = 'SOAPAction:' + '"urn:schemas-upnp-org:service:' + 'AVTransport' + ':1#' + 'GetMediaInfo"';

		var h = new HTTP(oCurlOptions, http_headers);

		h.reset();

		var url = 'http://' + szIpAddressRenderer + ':' + nPortRenderer + szServiceType;

		var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
				+ '<s:Body>' + '<u:GetMediaInfo xmlns:u="urn:schemas-upnp-org:service:AVTransport:1">' + '<InstanceID>0</InstanceID>' + '</u:GetMediaInfo>' + '</s:Body>' + '</s:Envelope>';

		var result = h.post(url, '', action);

		/* Set result status */
		oResult.status = result.status;
		oResult.data = upnpParseXMLRespond(result.body, 'CurrentURI', '');

		lInfo.logln("Extracted media content: " + oResult.data);
	} catch (e) {
		lWarn.logln("Error executing upnpGetMediaContent command " + e);
	}
}

/**
 * @brief Get upnp volume
 * 
 * @param nPlayerId -
 *            player ID
 * @param szIpAddressRenderer -
 *            IP address of the UPNP renderer
 * @param nPortRenderer -
 *            Port number of the UPNP renderer
 * @param szServiceType -
 *            Service type
 * @param oCurlOptions -
 *            additional options for curl used by HTTP
 * 
 * @return
 */
function upnpGetVolume(nPlayerId, szIpAddressRenderer, nPortRenderer, szServiceType, oResult, oCurlOptions) {

	try {
		var http_headers = [];
		http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
		http_headers['extra_hdr2'] = 'SOAPAction:' + '"urn:schemas-upnp-org:service:' + 'RenderingControl' + ':1#' + 'GetVolume"';

		var h = new HTTP(oCurlOptions, http_headers);

		h.reset();

		var url = 'http://' + szIpAddressRenderer + ':' + nPortRenderer + szServiceType;

		var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
				+ '<s:Body>' + '<u:GetVolume xmlns:u="urn:schemas-upnp-org:service:RenderingControl:1">' + '<InstanceID>0</InstanceID>' 
				+ '<Channel>Master</Channel>' + '</u:GetVolume>' + '</s:Body>' + '</s:Envelope>';

		var result = h.post(url, '', action);

		/* Set result status */
		oResult.status = result.status;
		oResult.data = upnpParseXMLRespond(result.body, 'CurrentVolume', '');

		
		lInfo.logln(url);
		lInfo.logln("Volume is " + oResult.data);
		return oResult.data;
	} catch (e) {
		lWarn.logln("Error executing upnpGetContentSrc command " + e);
	}
	return null;
}


/**
 * @brief Retrieve media info from the player and store it into the temp storage
 * 
 * @param nPlayerId -
 *            player ID
 * @param szIpAddressRenderer -
 *            IP address of the UPNP renderer
 * @param nPortRenderer -
 *            Port number of the UPNP renderer
 * @param szServiceType -
 *            Service type
 * @param oResult -
 *            holds result and returned data
 * @param oCurlOptions -
 *            additional options for curl used by HTTP
 * 
 * @return
 */
function upnpGetMediaInfo(nPlayerId, szIpAddressRenderer, nPortRenderer, szServiceType, oResult, oCurlOptions) {
	try {
		var http_headers = [];
		http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
		http_headers['extra_hdr2'] = 'SOAPAction:' + '"urn:schemas-upnp-org:service:' + 'AVTransport' + ':1#' + 'GetMediaInfo"';

		var h = new HTTP(oCurlOptions, http_headers);

		h.reset();

		var url = 'http://' + szIpAddressRenderer + ':' + nPortRenderer + szServiceType;

		var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
				+ '<s:Body>' + '<u:GetMediaInfo xmlns:u="urn:schemas-upnp-org:service:AVTransport:1">' + '<InstanceID>0</InstanceID>' + '</u:GetMediaInfo>' + '</s:Body>' + '</s:Envelope>';

		var result = h.post(url, '', action);

		var oReturn = new Object();
		oReturn.nrTracks = upnpParseXMLRespond(result.body, 'NrTracks', '');
		return oReturn;
	} catch (e) {
		lWarn.logln("Error executing upnpGetMediaInfo command " + e);
	}
	return null;
}

/*
 * function upnpLoadSonosPlayList(nPlayerId, szIpAddressRenderer, nPortRenderer ,
 * szServiceType, oResult, oCurlOptions) { try { var http_headers = [];
 * http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
 * http_headers['extra_hdr2'] = 'SOAPAction:' + '"urn:schemas-upnp-org:service:' +
 * 'AVTransport' + ':1#' + 'AddURIToQueue"';
 * 
 * var h = new HTTP(oCurlOptions, http_headers);
 * 
 * h.reset();
 * 
 * var url = 'http://' + szIpAddressRenderer + ':' + nPortRenderer +
 * szServiceType; var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope
 * s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"
 * xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">' + '<s:Body>' + '<u:AddURIToQueue
 * xmlns:u="urn:schemas-upnp-org:service:AVTransport:1">' + '<InstanceID>0</InstanceID>' + '<EnqueuedURI>file:///jffs/settings/savedqueues.rsq#12</EnqueuedURI>' + '<EnqueuedURIMetaData>&lt;DIDL-Lite
 * xmlns:dc=&quot;http://purl.org/dc/elements/1.1/&quot;
 * xmlns:upnp=&quot;urn:schemas-upnp-org:metadata-1-0/upnp/&quot;
 * xmlns:r=&quot;urn:schemas-rinconnetworks-com:metadata-1-0/&quot;
 * xmlns=&quot;urn:schemas-upnp-org:metadata-1-0/DIDL-Lite/&quot;&gt;&lt;item
 * id=&quot;SQ:12&quot; parentID=&quot;SQ:&quot;
 * restricted=&quot;true&quot;&gt;&lt;dc:title&gt;Amb Drum &amp;amp;
 * Bass&lt;/dc:title&gt;&lt;upnp:class&gt;object.container.pla
 * ylist&lt;/upnp:class&gt;&lt;desc id=&quot;cdudn&quot;
 * nameSpace=&quot;urn:schemas-rinconnetworks-com:metadata-1-0/&quot;&gt;RINCON_AssociatedZPUDN&lt;/desc&gt;&lt;/item&gt;&lt;/DIDL-Lite&gt;</EnqueuedURIMetaData>' + '<DesiredFirstTrackNumberEnqueued>0</DesiredFirstTrackNumberEnqueued>' + '<EnqueueAsNext>1</EnqueueAsNext>' + '</u:AddURIToQueue>' + '</s:Body>' + '</s:Envelope>'
 * 
 * var result = h.post(url, '', action);
 * 
 * return result; } catch (e) { lInfo.logln("Error executing
 * upnpLoadSonosPlayList command " + e); } } function
 * upnpSaveSonosPlayList(nPlayerId, szIpAddressRenderer, nPortRenderer ,
 * szServiceType, oResult, oCurlOptions) { try { var http_headers = [];
 * http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
 * http_headers['extra_hdr2'] = 'SOAPAction:' + '"urn:schemas-upnp-org:service:' +
 * 'AVTransport' + ':1#' + 'SaveQueue"';
 * 
 * var h = new HTTP(oCurlOptions, http_headers);
 * 
 * h.reset();
 * 
 * var url = 'http://' + szIpAddressRenderer + ':' + nPortRenderer +
 * szServiceType;
 * 
 * var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope
 * s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"
 * xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">' + '<s:Body>' + '<u:SaveQueue
 * xmlns:u="urn:schemas-upnp-org:service:AVTransport:1">' + '<InstanceID>0</InstanceID>' + '<Title>Test</Title>' + '<ObjectID></ObjectID>' + '</u:SaveQueue>' + '</s:Body>' + '</s:Envelope>'
 * 
 * var result = h.post(url, '', action);
 * 
 * return upnpParseXMLRespond(result.body, 'AssignedObjectID', '') } catch (e) {
 * lInfo.logln("Error executing upnpSaveSonosPlayList command " + e); } }
 * function upnpGetPlaylistSongs(nPlayerId, szIpAddressServer, nPortServer ,
 * sPlaylistCode, oCurlOptions) { try { var http_headers = [];
 * http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
 * http_headers['extra_hdr2'] = 'SOAPAction:' + '"urn:schemas-upnp-org:service:' +
 * 'ContentDirectory' + ':1#' + 'Browse"';
 * 
 * 
 * var h = new HTTP(oCurlOptions, http_headers);
 * 
 * h.reset();
 * 
 * var url = 'http://' + szIpAddressServer + ':' + nPortServer +
 * "/_urn-schemas-upnp-org-service-ContentDirectory_control"; var action = '<?xml
 * version="1.0" encoding="utf-8"?>' + '<s:Envelope
 * s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"
 * xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">' + '<s:Body>' + '<u:Browse
 * xmlns:u="urn:schemas-upnp-org:service:ContentDirectory:1">' + '<ObjectID>0000000000000005</ObjectID>' + '<BrowseFlag>BrowseDirectChildren</BrowseFlag>' + '<Filter>@childCount,dc:creator,upnp:album,upnp:artist,res,res@protection,res@duration,res@size,upnp:albumArtURI,upnp:searchClass</Filter>' + '<StartingIndex>0</StartingIndex>' + '<RequestedCount>0</RequestedCount>' + '<SortCriteria
 * />' + '</u:Browse>' + '</s:Body>' + '</s:Envelope>';
 * 
 * var result = h.post(url, '', action);
 * 
 * var oReturn = new Object(); oReturn.result = upnpParseXMLRespond(result.body,
 * 'Result', ''); oReturn.resultCount = upnpParseXMLRespond(result.body,
 * 'NumberReturned', '');
 * 
 * return oReturn; } catch (e) { lInfo.logln("Error executing
 * upnpGetPlaylistSongs command " + e); } }
 */

/**
 * @brief Retrieve position info from the player and store it into the temp
 *        storage
 * 
 * @param nPlayerId -
 *            player ID
 * @param szIpAddressRenderer -
 *            IP address of the UPNP renderer
 * @param nPortRenderer -
 *            Port number of the UPNP renderer
 * @param szServiceType -
 *            Service type
 * @param oResult -
 *            holds result and returned data
 * @param oCurlOptions -
 *            additional options for curl used by HTTP
 * 
 * @return
 */
function upnpGetPositionInfo(nPlayerId, szIpAddressRenderer, nPortRenderer, szServiceType, oResult, oCurlOptions) {
	try {
		var http_headers = [];
		http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
		http_headers['extra_hdr2'] = 'SOAPAction:' + '"urn:schemas-upnp-org:service:' + 'AVTransport' + ':1#' + 'GetPositionInfo"';

		var h = new HTTP(oCurlOptions, http_headers);

		h.reset();

		var url = 'http://' + szIpAddressRenderer + ':' + nPortRenderer + szServiceType;

		var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
				+ '<s:Body>' + '<u:GetPositionInfo xmlns:u="urn:schemas-upnp-org:service:AVTransport:1">' + '<InstanceID>0</InstanceID>' + '</u:GetPositionInfo>' + '</s:Body>' + '</s:Envelope>';

		var result = h.post(url, '', action);

		var oReturn = new Object();
		oReturn.track = upnpParseXMLRespond(result.body, 'TrackURI', '');
		oReturn.position = upnpParseXMLRespond(result.body, 'RelTime', '');
		oReturn.trackNR = upnpParseXMLRespond(result.body, 'Track', '');

		return oReturn;
	} catch (e) {
		lWarn.logln("Error executing upnpGetPositionInfo command " + e);
	}
	return null;
}

function upnpGetPositionTrackUrl(nPlayerId, szIpAddressRenderer, nPortRenderer, szServiceType, oResult, oCurlOptions) {
	try {
		var http_headers = [];
		http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
		http_headers['extra_hdr2'] = 'SOAPAction:' + '"urn:schemas-upnp-org:service:' + 'AVTransport' + ':1#' + 'GetPositionInfo"';

		var h = new HTTP(oCurlOptions, http_headers);

		h.reset();

		var url = 'http://' + szIpAddressRenderer + ':' + nPortRenderer + szServiceType;

		var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
				+ '<s:Body>' + '<u:GetPositionInfo xmlns:u="urn:schemas-upnp-org:service:AVTransport:1">' + '<InstanceID>0</InstanceID>' + '</u:GetPositionInfo>' + '</s:Body>' + '</s:Envelope>';

		var result = h.post(url, '', action);

		oResult.status = result.status;
		oResult.data = upnpParseXMLRespond(result.body, 'TrackURI', '');
		lInfo.logln("media info url: " + oResult.data);

	} catch (e) {
		lWarn.logln("Error executing upnpGetPositionInfo command " + e);
	}
}

/**
 * @brief Retrieve transport info from the player and store it into the temp
 *        storage
 * 
 * @param nPlayerId -
 *            player ID
 * @param szIpAddressRenderer -
 *            IP address of the UPNP renderer
 * @param nPortRenderer -
 *            Port number of the UPNP renderer
 * @param szServiceType -
 *            Service type
 * @param oResult -
 *            holds result and returned data
 * @param oCurlOptions -
 *            additional options for curl used by HTTP
 * 
 * @return
 */
function upnpGetTransportInfo(nPlayerId, szIpAddressRenderer, nPortRenderer, szServiceType, oResult, oCurlOptions) {
	try {
		var http_headers = [];
		http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
		http_headers['extra_hdr2'] = 'SOAPAction:' + '"urn:schemas-upnp-org:service:' + 'AVTransport' + ':1#' + 'GetTransportInfo"';

		var h = new HTTP(oCurlOptions, http_headers);

		h.reset();

		var url = 'http://' + szIpAddressRenderer + ':' + nPortRenderer + szServiceType;

		var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
				+ '<s:Body>' + '<u:GetTransportInfo xmlns:u="urn:schemas-upnp-org:service:AVTransport:1">' + '<InstanceID>0</InstanceID>' + '</u:GetTransportInfo>' + '</s:Body>' + '</s:Envelope>';

		var result = h.post(url, '', action);

		return upnpParseXMLRespond(result.body, 'CurrentTransportState', '')
	} catch (e) {
		lWarn.logln("Error executing upnpGetTransportInfo command " + e);
	}
	return null;
}

/**
 * @brief Retrieve transport settings from the player and store it into the temp
 *        storage
 * 
 * @param nPlayerId -
 *            player ID
 * @param szIpAddressRenderer -
 *            IP address of the UPNP renderer
 * @param nPortRenderer -
 *            Port number of the UPNP renderer
 * @param szServiceType -
 *            Service type
 * @param oResult -
 *            holds result and returned data
 * @param oCurlOptions -
 *            additional options for curl used by HTTP
 * 
 * @return
 */
function upnpGetTransportSettings(nPlayerId, szIpAddressRenderer, nPortRenderer, szServiceType, oResult, oCurlOptions) {
	try {
		var http_headers = [];
		http_headers['extra_hrd1'] = 'Content-Type: text/xml; charset="utf-8"';
		http_headers['extra_hdr2'] = 'SOAPAction:' + '"urn:schemas-upnp-org:service:' + 'AVTransport' + ':1#' + 'GetTransportSettings"';

		var h = new HTTP(oCurlOptions, http_headers);

		h.reset();

		var url = 'http://' + szIpAddressRenderer + ':' + nPortRenderer + szServiceType;

		var action = '<?xml version="1.0" encoding="utf-8"?>' + '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
				+ '<s:Body>' + '<u:GetTransportSettings xmlns:u="urn:schemas-upnp-org:service:AVTransport:1">' + '<InstanceID>0</InstanceID>' + '</u:GetTransportSettings>' + '</s:Body>'
				+ '</s:Envelope>';

		var result = h.post(url, '', action);
	} catch (e) {
		lWarn.logln("Error executing upnpGetTransportSettings command " + e);
	}
}

